﻿using UnityEngine;
using UnityEditor;
using System.IO;
using System.Text;

public class change
{
    // 添加一个右键菜单。
    // % 按下ctrl时显示菜单。（Windows: control, macOS: command）
    // & 按下alt时显示菜单。(Windows/Linux: alt, macOS: option)
    // _ 按下shift时显示菜单。(Windows/Linux/macOS: shift)
    [MenuItem("Assets/脚本改格式：GB2312->UTF8无BOM %g", false, 100)]
    private static void CustomMenu()
    {
            Object[] selectedObjects = Selection.objects;
            for(int i=0;i<selectedObjects.Length;i++)
            {

                // 例如: 获取Project视图中选定的对象
                Object selectedObject = selectedObjects[i];
                if (selectedObject != null)
                {
                    // 获取选定对象的相对路径
                    string relativeAssetPath = AssetDatabase.GetAssetPath(selectedObject);
                    // 获取项目根目录路径
                    string projectPath = Path.GetDirectoryName(Application.dataPath);
                    // 获取选定对象的绝对路径
                    string absoluteAssetPath = Path.Combine(projectPath, relativeAssetPath);
                    // 获取选定对象的文件名（包括后缀）
                    string fileName = Path.GetFileName(relativeAssetPath);

                    Debug.Log("执行自定义操作: " + selectedObject.name +
                            ", 相对路径: " + relativeAssetPath +
                            ", 绝对路径: " + absoluteAssetPath +
                            ", 文件名: " + fileName);

                    //判断是否是CSharp文件
                    if (IsCSharpFile(fileName))
                    {
                        Debug.Log("这是一个csharp文件");
                        ChangeFormat(absoluteAssetPath);
                    }
                    else
                    {
                        Debug.Log("兄弟，这不是一个csharp文件啊~~~~~~~~~~~");
                    }
                }
            }
    }

    // 如果项目视图中有选中的对象，则启用右键菜单项
    [MenuItem("Assets/脚本改格式：GB2312->UTF8无BOM %g", true)]
    private static bool ValidateCustomMenu()
    {
        return Selection.activeObject != null;
    }

    /// <summary>
    /// 判断该文件是否是CSharp文件
    /// </summary>
    /// <param name="fileName"></param>
    /// <returns></returns>
    private static bool IsCSharpFile(string fileName)
    {
        // 获取文件扩展名（包括点）
        string fileExtension = Path.GetExtension(fileName);

        // 将扩展名转换为小写并与 ".cs" 进行比较
        if (fileExtension.ToLower() == ".cs")
        {
            return true;
        }

        return false;
    }

    /// <summary>
    /// 文件格式转码：GB2312转成UTF8
    /// 读取指定的文件，转换成UTF8（无BOM标记）格式后，回写覆盖原文件
    /// </summary>
    /// <param name="sourceFilePath">文件路径</param>
    public static void ChangeFormat(string sourceFilePath)
    {
        Encoding ed =GetType(sourceFilePath);
        
        if( ed is System.Text.ASCIIEncoding )
        {
            ed =Encoding.GetEncoding("GB2312");
            string fileContent = File.ReadAllText(sourceFilePath, ed);
            File.WriteAllText(sourceFilePath, fileContent, Encoding.UTF8);
        }
        Debug.Log("处理结束！");
    }
    public static System.Text.Encoding GetType(string FILE_NAME)
    {
        FileStream fs = new FileStream(FILE_NAME, FileMode.Open, FileAccess.Read);
        Encoding r = GetType(fs);
        fs.Close();
        return r;
    }
        /// <summary>
    /// 通过给定的文件流，判断文件的编码类型
    /// </summary>
    /// <param name="fs">文件流</param>
    /// <returns>文件的编码类型</returns>
    public static System.Text.Encoding GetType(FileStream fs)
    {
        byte[] Unicode = new byte[] { 0xFF, 0xFE, 0x41 };
        byte[] UnicodeBIG = new byte[] { 0xFE, 0xFF, 0x00 };
        byte[] UTF8 = new byte[] { 0xEF, 0xBB, 0xBF }; //带BOM
        Encoding reVal = Encoding.Default;
 
        BinaryReader r = new BinaryReader(fs, System.Text.Encoding.Default);
        int i;
        int.TryParse(fs.Length.ToString(), out i);
        byte[] ss = r.ReadBytes(i);
        if (IsUTF8Bytes(ss) || (ss[0] == 0xEF && ss[1] == 0xBB && ss[2] == 0xBF))
        {
            reVal = Encoding.UTF8;
        }
        else if (ss[0] == 0xFE && ss[1] == 0xFF && ss[2] == 0x00)
        {
            reVal = Encoding.BigEndianUnicode;
        }
        else if (ss[0] == 0xFF && ss[1] == 0xFE && ss[2] == 0x41)
        {
            reVal = Encoding.Unicode;
        }
        r.Close();
        return reVal;
 
    }
      /// <summary>
    /// 判断是否是不带 BOM 的 UTF8 格式
    /// </summary>
    /// <param name="data"></param>
    /// <returns></returns>
    private static bool IsUTF8Bytes(byte[] data)
    {
        int charByteCounter = 1;
        //计算当前正分析的字符应还有的字节数
        byte curByte; //当前分析的字节.
        for (int i = 0; i < data.Length; i++)
        {
            curByte = data[i];
            if (charByteCounter == 1)
            {
                if (curByte >= 0x80)
                {
                    //判断当前
                    while (((curByte <<= 1) & 0x80) != 0)
                    {
                        charByteCounter++;
                    }
                    //标记位首位若为非0 则至少以2个1开始 如:110XXXXX...........1111110X
                    if (charByteCounter == 1 || charByteCounter > 6)
                    {
                        return false;
                    }
                }
            }
            else
            {
                //若是UTF-8 此时第一位必须为1
                if ((curByte & 0xC0) != 0x80)
                {
                    return false;
                }
                charByteCounter--;
            }
        }
        if (charByteCounter > 1)
        {
           // throw new Exception("非预期的byte格式");
        }
        return true;
    }
}
